@@ -99,11 +99,6 @@ public class CollectTargetedPcrMetrics extends CollectTargetedMetrics<TargetedPc
         return CUSTOM_AMPLICON_SET_NAME != null ? CUSTOM_AMPLICON_SET_NAME : CollectTargetedMetrics.renderProbeNameFromFile(AMPLICON_INTERVALS);
     }
 
-    /** Stock main method. */
-    public static void main(final String[] argv) {
-        System.exit(new CollectTargetedPcrMetrics().instanceMain(argv));
-    }
-
     @Override
     protected TargetedPcrMetricsCollector makeCollector(final Set<MetricAccumulationLevel> accumulationLevels,
                                                         final List<SAMReadGroupRecord> samRgRecords,
@@ -115,6 +110,6 @@ public class CollectTargetedPcrMetrics extends CollectTargetedMetrics<TargetedPc
                                                         final String probeSetName,
                                                         final int nearProbeDistance) {
         return new TargetedPcrMetricsCollector(accumulationLevels, samRgRecords, refFile, perTargetCoverage, perBaseCoverage, targetIntervals, probeIntervals, probeSetName, nearProbeDistance,
-                MINIMUM_MAPPING_QUALITY, MINIMUM_BASE_QUALITY, CLIP_OVERLAPPING_READS, true, COVERAGE_CAP, SAMPLE_SIZE);
+                MINIMUM_MAPPING_QUALITY, MINIMUM_BASE_QUALITY, CLIP_OVERLAPPING_READS, true, INCLUDE_INDELS, COVERAGE_CAP, SAMPLE_SIZE);
     }
 }
