@@ -33,6 +33,7 @@ import htsjdk.samtools.util.IOUtil;
 import htsjdk.samtools.util.Log;
 import org.broadinstitute.barclay.argparser.Argument;
 import org.broadinstitute.barclay.argparser.CommandLineProgramProperties;
+import org.broadinstitute.barclay.argparser.DeprecatedFeature;
 import org.broadinstitute.barclay.argparser.Hidden;
 import org.broadinstitute.barclay.help.DocumentedFeature;
 import picard.PicardException;
@@ -53,9 +54,8 @@ import java.nio.file.Path;
 import java.text.NumberFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.HashMap;
-import java.util.HashSet;
 import java.util.LinkedHashMap;
+import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -384,6 +384,7 @@ public class CrosscheckFingerprints extends CommandLineProgram {
             "marking has been overly aggressive and coverage is low.")
     public boolean ALLOW_DUPLICATE_READS = false;
 
+    @DeprecatedFeature(detail = "No longer used in fingerprint checking. Will be removed in a future release.")
     @Argument(doc = "Assumed genotyping error rate that provides a floor on the probability that a genotype comes from " +
             "the expected sample. Must be greater than zero. ")
     public double GENOTYPING_ERROR_RATE = 0.01;
@@ -423,12 +424,6 @@ public class CrosscheckFingerprints extends CommandLineProgram {
 
     @Override
     protected String[] customCommandLineValidation() {
-        if (GENOTYPING_ERROR_RATE <= 0) {
-            return new String[]{"GENOTYPING_ERROR_RATE must be greater than zero. Found " + GENOTYPING_ERROR_RATE};
-        }
-        if (GENOTYPING_ERROR_RATE >= 1) {
-            return new String[]{"GENOTYPING_ERROR_RATE must be strictly less than 1, found " + GENOTYPING_ERROR_RATE};
-        }
         if (SECOND_INPUT == null && INPUT_SAMPLE_MAP != null) {
             return new String[]{"INPUT_SAMPLE_MAP can only be used when also using SECOND_INPUT"};
         }
@@ -649,8 +644,8 @@ public class CrosscheckFingerprints extends CommandLineProgram {
         });
 
         if (CollectionUtil.makeSet(resultingSamples.toArray(new String[0])).size() != resultingSamples.size()) {
-            final Set<String> duplicates = new HashSet<>();
-            final Set<String> unique = new HashSet<>();
+            final Set<String> duplicates = new LinkedHashSet<>();
+            final Set<String> unique = new LinkedHashSet<>();
             resultingSamples.forEach(s -> {
                 if (unique.add(s)) {
                     duplicates.add(s);
@@ -662,7 +657,7 @@ public class CrosscheckFingerprints extends CommandLineProgram {
         }
 
         // replace samples with their mapped values:
-        final Set<FingerprintIdDetails> ids = new HashSet<>(fpMap.keySet());
+        final Set<FingerprintIdDetails> ids = new LinkedHashSet<>(fpMap.keySet());
         ids.forEach(id -> {
             // if sample isn't in sampleMap, leave it alone
             if (!sampleMap.containsKey(id.sample)) {
@@ -703,7 +698,7 @@ public class CrosscheckFingerprints extends CommandLineProgram {
 
         final Map<String, List<FingerprintIdDetails>> fileFpDetailSetMap = fpMap.keySet().stream().collect(Collectors.groupingBy(s -> s.file));
 
-        final Map<String, String> fileSampleMap = new HashMap<>();
+        final Map<String, String> fileSampleMap = new LinkedHashMap<>();
         // check that each file in the map points only to one sample
         fileFpDetailSetMap.forEach((key, fingerprintIdDetails) -> {
             final Set<String> samples = fingerprintIdDetails.stream().map(id -> id.sample).collect(Collectors.toSet());
@@ -725,8 +720,8 @@ public class CrosscheckFingerprints extends CommandLineProgram {
         });
 
         if (CollectionUtil.makeSet(resultingSamples.toArray(new String[0])).size() != resultingSamples.size()) {
-            final Set<String> duplicates = new HashSet<>();
-            final Set<String> unique = new HashSet<>();
+            final Set<String> duplicates = new LinkedHashSet<>();
+            final Set<String> unique = new LinkedHashSet<>();
             resultingSamples.forEach(s -> {
                 if (unique.add(s)) {
                     duplicates.add(s);
@@ -738,7 +733,7 @@ public class CrosscheckFingerprints extends CommandLineProgram {
         }
 
         // replace samples with their mapped values:
-        final Set<FingerprintIdDetails> ids = new HashSet<>(fpMap.keySet());
+        final Set<FingerprintIdDetails> ids = new LinkedHashSet<>(fpMap.keySet());
         ids.forEach(id -> {
             // if sample isn't in sampleMap, leave it alone
             if (!sampleMap.containsKey(id.file)) {
@@ -776,7 +771,7 @@ public class CrosscheckFingerprints extends CommandLineProgram {
 
     private Map<Path, Path> getSamplePathToIndexMap(final File indexMapFile, final String inputArgumentName) {
         final Map<String, String> indexStringMap = getStringStringMap(indexMapFile, inputArgumentName);
-        final HashMap<Path, Path> indexPathMap = new LinkedHashMap<Path, Path>();
+        final LinkedHashMap<Path, Path> indexPathMap = new LinkedHashMap<Path, Path>();
         for (Map.Entry<String, String> entry: indexStringMap.entrySet()) {
             final Path inputPath;
             final Path indexPath;
@@ -900,7 +895,7 @@ public class CrosscheckFingerprints extends CommandLineProgram {
                 final boolean expectedToMatch = EXPECT_ALL_GROUPS_TO_MATCH || lhsMatchId.equals(rhsMatchId);
 
                 final MatchResults results = FingerprintChecker.calculateMatchResults(lhsFingerprints.get(lhsId), rhsFingerprints.get(rhsId),
-                        GENOTYPING_ERROR_RATE, LOSS_OF_HET_RATE, false, CALCULATE_TUMOR_AWARE_RESULTS);
+                        LOSS_OF_HET_RATE, false, CALCULATE_TUMOR_AWARE_RESULTS);
                 final FingerprintResult result = getMatchResults(expectedToMatch, results);
 
                 if (!OUTPUT_ERRORS_ONLY || result == FingerprintResult.INCONCLUSIVE || !result.isExpected()) {
@@ -935,7 +930,7 @@ public class CrosscheckFingerprints extends CommandLineProgram {
         final Map<String, FingerprintIdDetails> sampleToDetail1 = fingerprints1BySample.keySet().stream().collect(Collectors.toMap(id -> id.group, id -> id));
         final Map<String, FingerprintIdDetails> sampleToDetail2 = fingerprints2BySample.keySet().stream().collect(Collectors.toMap(id -> id.group, id -> id));
 
-        Set<String> samples = new HashSet<>();
+        Set<String> samples = new LinkedHashSet<>();
         samples.addAll(sampleToDetail1.keySet());
         samples.addAll(sampleToDetail2.keySet());
 
@@ -958,7 +953,7 @@ public class CrosscheckFingerprints extends CommandLineProgram {
             }
 
             final MatchResults results = FingerprintChecker.calculateMatchResults(lhsFP, rhsFP,
-                    GENOTYPING_ERROR_RATE, LOSS_OF_HET_RATE, false, CALCULATE_TUMOR_AWARE_RESULTS);
+                    LOSS_OF_HET_RATE, false, CALCULATE_TUMOR_AWARE_RESULTS);
             final CrosscheckMetric.FingerprintResult result = getMatchResults(true, results);
 
             if (!OUTPUT_ERRORS_ONLY || !result.isExpected()) {
