@@ -1,27 +1,11 @@
 package picard.sam;
 
-import htsjdk.samtools.BAMRecordCodec;
-import htsjdk.samtools.CigarElement;
-import htsjdk.samtools.CigarOperator;
-import htsjdk.samtools.MergingSamRecordIterator;
-import htsjdk.samtools.SAMFileHeader;
+import htsjdk.samtools.*;
 import htsjdk.samtools.SAMFileHeader.SortOrder;
-import htsjdk.samtools.SAMProgramRecord;
-import htsjdk.samtools.SAMRecord;
-import htsjdk.samtools.SAMRecordQueryNameComparator;
-import htsjdk.samtools.SamFileHeaderMerger;
-import htsjdk.samtools.SamPairUtil;
-import htsjdk.samtools.SamReader;
-import htsjdk.samtools.SamReaderFactory;
-import htsjdk.samtools.ValidationStringency;
 import htsjdk.samtools.filter.OverclippedReadFilter;
-import htsjdk.samtools.util.CloseableIterator;
-import htsjdk.samtools.util.CloserUtil;
-import htsjdk.samtools.util.DelegatingIterator;
-import htsjdk.samtools.util.IOUtil;
-import htsjdk.samtools.util.Log;
-import htsjdk.samtools.util.PeekableIterator;
-import htsjdk.samtools.util.SortingCollection;
+import htsjdk.samtools.util.*;
+import htsjdk.variant.utils.SAMSequenceDictionaryExtractor;
+import picard.PicardException;
 
 import java.io.File;
 import java.util.ArrayList;
@@ -47,50 +31,12 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
     private final int minUnclippedBases;
     private boolean forceSort = false;
     private final OverclippedReadFilter contaminationFilter;
+    private final List<String> requiredMatchingDictionaryTags;
+    private SAMSequenceDictionary alignedSamDictionary;
 
     /**
-     * Constructor
+     *  Constructor with a default value for unmappingReadStrategy
      *
-     * @param unmappedBamFile                   The BAM file that was used as the input to the aligner, which will
-     *                                          include info on all the reads that did not map.  Required.
-     * @param targetBamFile                     The file to which to write the merged SAM records. Required.
-     * @param referenceFasta                    The reference sequence for the map files. Required.
-     * @param programRecord                     Program record for taget file SAMRecords created.
-     * @param clipAdapters                      Whether adapters marked in unmapped BAM file should be marked as
-     *                                          soft clipped in the merged bam. Required.
-     * @param bisulfiteSequence                 Whether the reads are bisulfite sequence (used when calculating the
-     *                                          NM and UQ tags). Required.
-     * @param alignedReadsOnly                  Whether to output only those reads that have alignment data
-     * @param alignedSamFile                    The SAM file(s) with alignment information.  Optional.  If this is
-     *                                          not provided, then read1AlignedSamFile and read2AlignedSamFile must be.
-     * @param maxGaps                           The maximum number of insertions or deletions permitted in an
-     *                                          alignment.  Alignments with more than this many gaps will be ignored.
-     *                                          -1 means to allow any number of gaps.
-     * @param attributesToRetain                attributes from the alignment record that should be
-     *                                          retained when merging, overridden by attributesToRemove if they share
-     *                                          common tags.
-     * @param attributesToRemove                attributes from the alignment record that should be
-     *                                          removed when merging.  This overrides attributesToRetain if they share
-     *                                          common tags.
-     * @param read1BasesTrimmed                 The number of bases trimmed from start of read 1 prior to alignment.  Optional.
-     * @param read2BasesTrimmed                 The number of bases trimmed from start of read 2 prior to alignment.  Optional.
-     * @param read1AlignedSamFile               The alignment records for read1.  Used when the two ends of a read are
-     *                                          aligned separately.  This is optional, but must be specified if
-     *                                          alignedSamFile is not.
-     * @param read2AlignedSamFile               The alignment records for read1.  Used when the two ends of a read are
-     *                                          aligned separately.  This is optional, but must be specified if
-     *                                          alignedSamFile is not.
-     * @param expectedOrientations              A List of SamPairUtil.PairOrientations that are expected for
-     *                                          aligned pairs.  Used to determine the properPair flag.
-     * @param sortOrder                         The order in which the merged records should be output.  If null,
-     *                                          output will be coordinate-sorted
-     * @param primaryAlignmentSelectionStrategy How to handle multiple alignments for a fragment or read pair,
-     *                                          in which none are primary, or more than one is marked primary
-     * @param addMateCigar                      True if we are to add or maintain the mate CIGAR (MC) tag, false if we are to remove or not include.
-     *
-     * @param unmapContaminantReads             If true, identify reads having the signature of contamination from a foreign organism (i.e. mostly clipped bases),
-     *                                          and mark them as unmapped.
-     * @param minUnclippedBases                 If unmapContaminantReads is set, require this many unclipped bases or else the read will be marked as contaminant.
      */
     public SamAlignmentMerger(final File unmappedBamFile, final File targetBamFile, final File referenceFasta,
                               final SAMProgramRecord programRecord, final boolean clipAdapters, final boolean bisulfiteSequence,
@@ -105,10 +51,99 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
                               final boolean addMateCigar,
                               final boolean unmapContaminantReads,
                               final int minUnclippedBases) {
+        this(unmappedBamFile,
+                targetBamFile,
+                referenceFasta,
+                programRecord,
+                clipAdapters,
+                bisulfiteSequence,
+                alignedReadsOnly,
+                alignedSamFile,
+                maxGaps,
+                attributesToRetain,
+                attributesToRemove,
+                read1BasesTrimmed,
+                read2BasesTrimmed,
+                read1AlignedSamFile,
+                read2AlignedSamFile,
+                expectedOrientations,
+                sortOrder,
+                primaryAlignmentSelectionStrategy,
+                addMateCigar,
+                unmapContaminantReads,
+                minUnclippedBases,
+                UnmappingReadStrategy.DO_NOT_CHANGE,
+                SAMSequenceDictionary.DEFAULT_DICTIONARY_EQUAL_TAG
+                );
+    }
+
+        /**
+         * Constructor
+         *
+         * @param unmappedBamFile                   The BAM file that was used as the input to the aligner, which will
+         *                                          include info on all the reads that did not map.  Required.
+         * @param targetBamFile                     The file to which to write the merged SAM records. Required.
+         * @param referenceFasta                    The reference sequence for the map files. Required.
+         * @param programRecord                     Program record for taget file SAMRecords created.
+         * @param clipAdapters                      Whether adapters marked in unmapped BAM file should be marked as
+         *                                          soft clipped in the merged bam. Required.
+         * @param bisulfiteSequence                 Whether the reads are bisulfite sequence (used when calculating the
+         *                                          NM and UQ tags). Required.
+         * @param alignedReadsOnly                  Whether to output only those reads that have alignment data
+         * @param alignedSamFile                    The SAM file(s) with alignment information.  Optional.  If this is
+         *                                          not provided, then read1AlignedSamFile and read2AlignedSamFile must be.
+         * @param maxGaps                           The maximum number of insertions or deletions permitted in an
+         *                                          alignment.  Alignments with more than this many gaps will be ignored.
+         *                                          -1 means to allow any number of gaps.
+         * @param attributesToRetain                attributes from the alignment record that should be
+         *                                          retained when merging, overridden by attributesToRemove if they share
+         *                                          common tags.
+         * @param attributesToRemove                attributes from the alignment record that should be
+         *                                          removed when merging.  This overrides attributesToRetain if they share
+         *                                          common tags.
+         * @param read1BasesTrimmed                 The number of bases trimmed from start of read 1 prior to alignment.  Optional.
+         * @param read2BasesTrimmed                 The number of bases trimmed from start of read 2 prior to alignment.  Optional.
+         * @param read1AlignedSamFile               The alignment records for read1.  Used when the two ends of a read are
+         *                                          aligned separately.  This is optional, but must be specified if
+         *                                          alignedSamFile is not.
+         * @param read2AlignedSamFile               The alignment records for read1.  Used when the two ends of a read are
+         *                                          aligned separately.  This is optional, but must be specified if
+         *                                          alignedSamFile is not.
+         * @param expectedOrientations              A List of SamPairUtil.PairOrientations that are expected for
+         *                                          aligned pairs.  Used to determine the properPair flag.
+         * @param sortOrder                         The order in which the merged records should be output.  If null,
+         *                                          output will be coordinate-sorted
+         * @param primaryAlignmentSelectionStrategy How to handle multiple alignments for a fragment or read pair,
+         *                                          in which none are primary, or more than one is marked primary
+         * @param addMateCigar                      True if we are to add or maintain the mate CIGAR (MC) tag, false if we are to remove or not include.
+         * @param unmapContaminantReads             If true, identify reads having the signature of cross-species contamination (i.e. mostly clipped bases),
+         *                                          and mark them as unmapped.
+         * @param minUnclippedBases                 If unmapContaminantReads is set, require this many unclipped bases or else the read will be marked as contaminant.
+         * @param unmappingReadStrategy             An enum describing how to deal with reads whose mapping information are being removed (currently this happens due to cross-species
+         *                                          contamination). Ignored unless unmapContaminantReads is true.
+         * @param requiredMatchingDictionaryTags            A list of SAMSequenceRecord tags that must be equal (if present) in the aligned bam and the reference dictionary.
+         *                                          Program will issue a warning about other tags, if present in both files and are different.
+         */
+    public SamAlignmentMerger(final File unmappedBamFile, final File targetBamFile, final File referenceFasta,
+                              final SAMProgramRecord programRecord, final boolean clipAdapters, final boolean bisulfiteSequence,
+                              final boolean alignedReadsOnly,
+                              final List<File> alignedSamFile, final int maxGaps, final List<String> attributesToRetain,
+                              final List<String> attributesToRemove,
+                              final Integer read1BasesTrimmed, final Integer read2BasesTrimmed,
+                              final List<File> read1AlignedSamFile, final List<File> read2AlignedSamFile,
+                              final List<SamPairUtil.PairOrientation> expectedOrientations,
+                              final SortOrder sortOrder,
+                              final PrimaryAlignmentSelectionStrategy primaryAlignmentSelectionStrategy,
+                              final boolean addMateCigar,
+                              final boolean unmapContaminantReads,
+                              final int minUnclippedBases,
+                              final UnmappingReadStrategy unmappingReadStrategy,
+                              final List<String> requiredMatchingDictionaryTags) {
 
         super(unmappedBamFile, targetBamFile, referenceFasta, clipAdapters, bisulfiteSequence,
                 alignedReadsOnly, programRecord, attributesToRetain, attributesToRemove, read1BasesTrimmed,
-                read2BasesTrimmed, expectedOrientations, sortOrder, primaryAlignmentSelectionStrategy, addMateCigar, unmapContaminantReads);
+                read2BasesTrimmed, expectedOrientations, sortOrder, primaryAlignmentSelectionStrategy, addMateCigar,
+                unmapContaminantReads, unmappingReadStrategy);
 
         if ((alignedSamFile == null || alignedSamFile.isEmpty()) &&
                 (read1AlignedSamFile == null || read1AlignedSamFile.isEmpty() ||
@@ -118,16 +153,10 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
         }
 
         if (alignedSamFile != null) {
-            for (final File f : alignedSamFile) {
-                IOUtil.assertFileIsReadable(f);
-            }
+            alignedSamFile.forEach(IOUtil::assertFileIsReadable);
         } else {
-            for (final File f : read1AlignedSamFile) {
-                IOUtil.assertFileIsReadable(f);
-            }
-            for (final File f : read2AlignedSamFile) {
-                IOUtil.assertFileIsReadable(f);
-            }
+            read1AlignedSamFile.forEach(IOUtil::assertFileIsReadable);
+            read2AlignedSamFile.forEach(IOUtil::assertFileIsReadable);
         }
 
         this.alignedSamFile = alignedSamFile;
@@ -136,11 +165,11 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
         this.maxGaps = maxGaps;
         this.minUnclippedBases = minUnclippedBases;
         this.contaminationFilter = new OverclippedReadFilter(minUnclippedBases, false);
+        this.requiredMatchingDictionaryTags = requiredMatchingDictionaryTags;
 
-        log.info("Processing SAM file(s): " + alignedSamFile != null ? alignedSamFile : read1AlignedSamFile + "," + read2AlignedSamFile);
+        log.info("Processing SAM file(s): " + ((alignedSamFile != null) ? alignedSamFile : (read1AlignedSamFile + "," + read2AlignedSamFile)));
     }
 
-
     /**
      * Merges the alignment from the map file with the non-aligned records from the source BAM file.
      * Overrides mergeAlignment in AbstractAlignmentMerger.  Tries first to proceed on the assumption
@@ -158,6 +187,16 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
         }
     }
 
+    @Override
+    protected SAMSequenceDictionary getDictionaryForMergedBam() {
+        SAMSequenceDictionary referenceDict = SAMSequenceDictionaryExtractor.extractDictionary(referenceFasta.toPath());
+        if (referenceDict == null) {
+            throw new PicardException("No sequence dictionary found for " + referenceFasta.getAbsolutePath() +
+                    ".  Use Picard's CreateSequenceDictionary to create a sequence dictionary.");
+        }
+        return SAMSequenceDictionary.mergeDictionaries(alignedSamDictionary, referenceDict, requiredMatchingDictionaryTags);
+    }
+
     /**
      * Reads the aligned SAM records into a SortingCollection and returns an iterator over that collection
      */
@@ -168,8 +207,8 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
 
         // When the alignment records, including both ends of a pair, are in SAM files
         if (alignedSamFile != null && !alignedSamFile.isEmpty()) {
-            final List<SAMFileHeader> headers = new ArrayList<SAMFileHeader>(alignedSamFile.size());
-            final List<SamReader> readers = new ArrayList<SamReader>(alignedSamFile.size());
+            final List<SAMFileHeader> headers = new ArrayList<>(alignedSamFile.size());
+            final List<SamReader> readers = new ArrayList<>(alignedSamFile.size());
             for (final File f : this.alignedSamFile) {
                 final SamReader r = SamReaderFactory.makeDefault().referenceSequence(referenceFasta).open(f);
                 headers.add(r.getFileHeader());
@@ -182,6 +221,12 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
                 }
             }
 
+            // assert that all the dictionaries are the same before merging the headers.
+            alignedSamDictionary = headers.get(0).getSequenceDictionary();
+            headers.stream()
+                    .map(SAMFileHeader::getSequenceDictionary)
+                    .forEach(alignedSamDictionary::assertSameDictionary);
+
             final SamFileHeaderMerger headerMerger = new SamFileHeaderMerger(SortOrder.queryname, headers, false);
 
             mergingIterator = new MergingSamRecordIterator(headerMerger, readers, true);
@@ -191,9 +236,12 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
         // When the ends are aligned separately and don't have firstOfPair information correctly
         // set we use this branch.
         else {
+            // this merging iterator already asserts that all the headers are the same
             mergingIterator = new SeparateEndAlignmentIterator(this.read1AlignedSamFile, this.read2AlignedSamFile, referenceFasta);
             header = ((SeparateEndAlignmentIterator) mergingIterator).getHeader();
 
+            alignedSamDictionary = header.getSequenceDictionary();
+
             // As we're going through and opening the aligned files, if we don't have a @PG yet
             // and there is only a single one in the input file, use that!
             if (getProgramRecord() == null && header.getProgramRecords().size() == 1) {
@@ -201,12 +249,10 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
             }
         }
 
-
         if (!forceSort) {
             return mergingIterator;
         }
 
-
         final SortingCollection<SAMRecord> alignmentSorter = SortingCollection.newInstance(SAMRecord.class,
                 new BAMRecordCodec(header), new SAMRecordQueryNameComparator(), MAX_RECORDS_IN_RAM);
 
@@ -272,9 +318,9 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
         private final SAMFileHeader header;
 
         public SeparateEndAlignmentIterator(final List<File> read1Alignments, final List<File> read2Alignments, File referenceFasta) {
-            final List<SAMFileHeader> headers = new ArrayList<SAMFileHeader>();
-            final List<SamReader> read1 = new ArrayList<SamReader>(read1Alignments.size());
-            final List<SamReader> read2 = new ArrayList<SamReader>(read2Alignments.size());
+            final List<SAMFileHeader> headers = new ArrayList<>();
+            final List<SamReader> read1 = new ArrayList<>(read1Alignments.size());
+            final List<SamReader> read2 = new ArrayList<>(read2Alignments.size());
             for (final File f : read1Alignments) {
                 final SamReader r = SamReaderFactory.makeDefault().referenceSequence(referenceFasta).open(f);
                 headers.add(r.getFileHeader());
@@ -287,9 +333,9 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
             }
 
             final SamFileHeaderMerger headerMerger = new SamFileHeaderMerger(SAMFileHeader.SortOrder.coordinate, headers, false);
-            read1Iterator = new PeekableIterator<SAMRecord>(
+            read1Iterator = new PeekableIterator<>(
                     new SuffixTrimingSamRecordIterator(new MergingSamRecordIterator(headerMerger, read1, true), "/1"));
-            read2Iterator = new PeekableIterator<SAMRecord>(
+            read2Iterator = new PeekableIterator<>(
                     new SuffixTrimingSamRecordIterator(new MergingSamRecordIterator(headerMerger, read2, true), "/2"));
 
             header = headerMerger.getMergedHeader();
@@ -333,7 +379,7 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
     }
 
     /**
-     * For now, we only ignore those alignments that have more than <code>maxGaps</code> insertions
+     * For now, we ignore only those alignments that have more than {@link #maxGaps} insertions
      * or deletions.
      */
     protected boolean ignoreAlignment(final SAMRecord sam) {
@@ -351,7 +397,6 @@ public class SamAlignmentMerger extends AbstractAlignmentMerger {
      * Criteria for contaminant reads:
      * 1. primary alignment has fewer than minUnclippedBases unclipped bases
      * 2. primary alignment has both ends clipped
-     * 3. for pairs, at least one end of primary alignment meets above criteria
      */
     protected boolean isContaminant(final HitsForInsert hits) {
         boolean isContaminant = false;
