@@ -33,10 +33,10 @@ import htsjdk.samtools.util.Log;
 import htsjdk.samtools.util.SamLocusIterator.RecordAndOffset;
 import htsjdk.samtools.util.SequenceUtil;
 import org.broadinstitute.barclay.argparser.CommandLineParser;
-import picard.sam.markduplicates.util.OpticalDuplicateFinder;
 import picard.sam.util.Pair;
 import picard.sam.util.PhysicalLocation;
 import picard.sam.util.PhysicalLocationInt;
+import picard.sam.util.ReadNameParser;
 
 import java.util.Collection;
 import java.util.LinkedList;
@@ -58,6 +58,7 @@ public class ReadBaseStratification {
     // instances.
     private static int LONG_HOMOPOLYMER = 6;
     private static int GC_CACHE_SIZE = 1000;
+    private static int LOCATION_BIN_SIZE = 2500;
 
 
     /* ***** SETTERS ********** */
@@ -73,6 +74,13 @@ public class ReadBaseStratification {
         LONG_HOMOPOLYMER = longHomopolymer;
     }
 
+    /**
+     * defaults to 2500
+     **/
+    public static void setLocationBinSize(int locationBinSize) { 
+       LOCATION_BIN_SIZE = locationBinSize; 
+    }
+
     /* ******* general-use classes, for defining and creating new stratifiers ***********/
 
     /**
@@ -100,6 +108,20 @@ public class ReadBaseStratification {
         abstract T stratify(final SAMRecord sam);
     }
 
+    /**
+     * A simple position based stratifier for cases when only the record suffices
+     */
+    abstract static class PositionBasedStratifier implements RecordAndOffsetStratifier<Integer> {
+        @Override
+        public Integer stratify(RecordAndOffset recordAndOffset, SAMLocusAndReference locusInfo) {
+            return stratify(recordAndOffset.getRecord());
+        }
+        //Static ReadNameParser so that cache of read names/ PhysicalLocations is shared between all PositionBasedStratifiers
+        static final ReadNameParser readNameParser = new ReadNameParser();
+
+        abstract Integer stratify(final SAMRecord sam);
+    }
+
     /**
      * A factory for generating stateless stratifier instances given a static function and a string
      *
@@ -461,14 +483,13 @@ public class ReadBaseStratification {
     /**
      * Stratifies base into their read's tile which is parsed from the read-name.
      */
-    public static class FlowCellTileStratifier extends RecordStratifier<Integer> {
-        private static OpticalDuplicateFinder opticalDuplicateFinder = new OpticalDuplicateFinder();
+    public static class FlowCellTileStratifier extends PositionBasedStratifier {
 
         @Override
         public Integer stratify(final SAMRecord sam) {
             try {
                 final PhysicalLocation location = new PhysicalLocationInt();
-                opticalDuplicateFinder.addLocationInformation(sam.getReadName(), location);
+                readNameParser.addLocationInformation(sam.getReadName(), location);
                 return (int) location.getTile();
             } catch (final IllegalArgumentException ignored) {
                 return null;
@@ -481,6 +502,51 @@ public class ReadBaseStratification {
         }
     }
 
+    /**
+     * Stratifies base into their read's Y coordinate which is parsed from the read-name.
+     */
+    public static class FlowCellYStratifier extends PositionBasedStratifier {
+
+        @Override
+        public Integer stratify(final SAMRecord sam) {
+            try {
+                final PhysicalLocation location = new PhysicalLocationInt();
+                readNameParser.addLocationInformation(sam.getReadName(), location);
+                return location.getY() / LOCATION_BIN_SIZE;
+            } catch (final IllegalArgumentException ignored) {
+                return null;
+            }
+        }
+
+
+        @Override
+        public String getSuffix() {
+            return "y";
+        }
+    }
+
+    /**
+     * Stratifies base into their read's X coordinate which is parsed from the read-name.
+     */
+    public static class FlowCellXStratifier extends PositionBasedStratifier {
+
+        @Override
+        public Integer stratify(final SAMRecord sam) {
+            try {
+                final PhysicalLocation location = new PhysicalLocationInt();
+                readNameParser.addLocationInformation(sam.getReadName(), location);
+                return location.getX() / LOCATION_BIN_SIZE;
+            } catch (final IllegalArgumentException ignored) {
+                return null;
+            }
+        }
+
+        @Override
+        public String getSuffix() {
+            return "x";
+        }
+    }
+
     /**
      * Stratifies according to the number of matching cigar operators (from CIGAR string) that the read has.
      */
@@ -638,6 +704,17 @@ public class ReadBaseStratification {
      */
     public static final FlowCellTileStratifier flowCellTileStratifier = new FlowCellTileStratifier();
 
+    /**
+     * Stratifies base into their read's tile which is parsed from the read-name.
+     */
+    public static final FlowCellXStratifier flowCellXStratifier = new FlowCellXStratifier();
+
+    /**
+     * Stratifies base into their read's tile which is parsed from the read-name.
+     */
+    public static final FlowCellYStratifier flowCellYStratifier = new FlowCellYStratifier();
+
+
     /**
      * Stratifies to the readgroup id of the read.
      */
@@ -757,6 +834,8 @@ public class ReadBaseStratification {
         //using a lazy initializer to enable the value of LONG_HOMOPOLYMER to be used;
         BINNED_HOMOPOLYMER(binnedHomopolymerStratifier::get, "The scale of homopolymer (long or short), the base that the homopolymer is comprised of, and the reference base."),
         FLOWCELL_TILE(() -> flowCellTileStratifier, "The flowcell and tile where the base was read (taken from the read name)."),
+        FLOWCELL_Y(() -> flowCellYStratifier, "The y-coordinate of the read (taken from the read name)"),
+        FLOWCELL_X(() -> flowCellXStratifier, "The x-coordinate of the read (taken from the read name)"),
         READ_GROUP(() -> readgroupStratifier, "The read-group id of the read."),
         CYCLE(() -> baseCycleStratifier, "The machine cycle during which the base was read."),
         BINNED_CYCLE(() -> binnedReadCycleStratifier, "The binned machine cycle. Similar to CYCLE, but binned into 5 evenly spaced ranges across the size of the read.  This stratifier may produce confusing results when used on datasets with variable sized reads."),
@@ -1209,4 +1288,4 @@ public class ReadBaseStratification {
         }
         return null;
     }
-}
\ No newline at end of file
+}
