@@ -3,6 +3,7 @@ package picard.sam.util;
 import htsjdk.samtools.*;
 import htsjdk.samtools.metrics.Header;
 import htsjdk.samtools.metrics.MetricsFile;
+import htsjdk.samtools.util.Histogram;
 import htsjdk.samtools.util.IterableAdapter;
 import htsjdk.samtools.util.SortingCollection;
 import picard.PicardException;
@@ -29,6 +30,10 @@ public final class SamComparison {
 
     private final SAMComparisonArgumentCollection samComparisonArgumentCollection;
 
+    // Ideally, this should be a Histogram<Pair<Integer, Integer>>, however, the MetricsFile class cannot read
+    // arbitrary types, therefore, it must be converted to a String, which may be slower
+    private final Histogram<String> mappingQualityHistogram = new Histogram<>();
+
     private SortingCollection<SAMRecord> markDuplicatesCheckLeft;
     private SortingCollection<SAMRecord> markDuplicatesCheckRight;
 
@@ -65,10 +70,13 @@ public final class SamComparison {
     }
 
     public void writeReport(final File output, final List<Header> headers) {
-        final MetricsFile<SamComparisonMetric, ?> comparisonMetricFile = new MetricsFile<>();
+        final MetricsFile<SamComparisonMetric, String> comparisonMetricFile = new MetricsFile<>();
 
         headers.forEach(comparisonMetricFile::addHeader);
         comparisonMetricFile.addAllMetrics(Collections.singletonList(comparisonMetric));
+        if (samComparisonArgumentCollection.COMPARE_MQ) {
+            comparisonMetricFile.addHistogram(mappingQualityHistogram);
+        }
         comparisonMetricFile.write(output);
     }
 
@@ -397,6 +405,10 @@ public final class SamComparison {
                 (samComparisonArgumentCollection.LENIENT_UNKNOWN_MQ_ALIGNMENT && s1.getMappingQuality() == SAMRecord.UNKNOWN_MAPPING_QUALITY && s2.getMappingQuality() == SAMRecord.UNKNOWN_MAPPING_QUALITY));
     }
 
+    private void compareAndUpdateMappingQualityConcordance(final SAMRecord s1, final SAMRecord s2) {
+        mappingQualityHistogram.increment(String.format("%d,%d", s1.getMappingQuality(), s2.getMappingQuality()));
+    }
+
     /**
      * Compare the mapping information for two SAMRecords.  Makes comparison of alignments, and also catalogs duplicate marking differences.
      */
@@ -407,6 +419,10 @@ public final class SamComparison {
         catalogDuplicateDifferences(s1, s2);
         final AlignmentComparison comp = compareAlignmentRecords(s1, s2);
         comparisonMetric.updateMetric(comp);
+
+        if (samComparisonArgumentCollection.COMPARE_MQ) {
+            compareAndUpdateMappingQualityConcordance(s1, s2);
+        }
     }
 
     private void catalogDuplicateDifferences(final SAMRecord s1, final SAMRecord s2) {
