@@ -124,10 +124,6 @@ public class ScatterIntervalsByNs extends CommandLineProgram {
     private static final ProgressLogger locusProgress = new ProgressLogger(log, (int) 1e7, "examined", "loci");
     private static final ProgressLogger intervalProgress = new ProgressLogger(log, (int) 10, "found", "intervals");
 
-    public static void main(final String[] args) {
-        new ScatterIntervalsByNs().instanceMainWithExit(args);
-    }
-
     // return a custom argument collection since this tool uses a (required) argument name
     // of "REFERENCE", not "REFERENCE_SEQUENCE"
     @Override
