@@ -1,11 +1,13 @@
 package picard.vcf;
 
+import htsjdk.io.HtsPath;
 import htsjdk.variant.vcf.VCFFileReader;
 import org.testng.Assert;
 import org.testng.annotations.BeforeClass;
 import org.testng.annotations.DataProvider;
 import org.testng.annotations.Test;
 import picard.cmdline.CommandLineProgramTest;
+import picard.nio.PicardHtsPath;
 
 import java.io.File;
 import java.io.IOException;
@@ -24,23 +26,24 @@ public class GatherVcfsTest extends CommandLineProgramTest {
         return GatherVcfs.class.getSimpleName();
     }
 
-    private File vcf,shard1, shard2, shard3, shard2_bad;
-    private File vcf_gz,shard1_gz, shard2_gz, shard3_gz, shard2_bad_gz;
+    private File vcf, vcf_gz;
+    private PicardHtsPath shard1, shard2, shard3, shard2_bad;
+    private HtsPath shard1_gz, shard2_gz, shard3_gz, shard2_bad_gz;
 
     @BeforeClass
     public void setup() throws IOException {
         final File TEST_DIR = new File("testdata/picard/vcf/GatherVcf");
         vcf = VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"input.vcf"),"whole.");
-        shard1 = VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard1.vcf"),"shard1.");
-        shard2 = VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard2.vcf"),"shard2.");
-        shard2_bad = VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard2_bad.vcf"),"shard2_bad.");
-        shard3 = VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard3.vcf"),"shard3");
+        shard1 = new PicardHtsPath(VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard1.vcf"),"shard1."));
+        shard2 = new PicardHtsPath(VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard2.vcf"),"shard2."));
+        shard2_bad = new PicardHtsPath(VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard2_bad.vcf"),"shard2_bad."));
+        shard3 = new PicardHtsPath(VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard3.vcf"),"shard3"));
 
         vcf_gz = VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"input.vcf"),"whole.",".vcf.gz");
-        shard1_gz = VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard1.vcf"),"shard1.",".vcf.gz");
-        shard2_gz = VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard2.vcf"),"shard2.",".vcf.gz");
-        shard2_bad_gz = VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard2_bad.vcf"),"shard2_bad.",".vcf.gz");
-        shard3_gz = VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard3.vcf"),"shard3.",".vcf.gz");
+        shard1_gz = new PicardHtsPath(VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard1.vcf"),"shard1.",".vcf.gz"));
+        shard2_gz = new PicardHtsPath(VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard2.vcf"),"shard2.",".vcf.gz"));
+        shard2_bad_gz = new PicardHtsPath(VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard2_bad.vcf"),"shard2_bad.",".vcf.gz"));
+        shard3_gz = new PicardHtsPath(VcfTestUtils.createTemporaryIndexedVcfFromInput(new File(TEST_DIR,"shard3.vcf"),"shard3.",".vcf.gz"));
 
     }
 
@@ -61,18 +64,18 @@ public class GatherVcfsTest extends CommandLineProgramTest {
     }
 
     @Test(dataProvider = "vcfshards")
-    public void TestGatherFiles(final List<File> inputFiles, final File expectedOutput, final int expectedRetVal, boolean reorder) throws IOException {
+    public void TestGatherFiles(final List<HtsPath> inputFiles, final File expectedOutput, final int expectedRetVal, boolean reorder) throws IOException {
         final String comment1 = "This is a comment";
         final List<String> args = new ArrayList<>();
 
         final File output = VcfTestUtils.createTemporaryIndexedFile("result", expectedOutput.getAbsolutePath().endsWith(".vcf") ? ".vcf" : ".vcf.gz");
 
-        inputFiles.forEach(f -> args.add("INPUT=" + f.getAbsolutePath()));
+        inputFiles.forEach(f -> args.add("INPUT=" + f.getURI()));
         args.add("OUTPUT=" + output.getAbsolutePath());
         args.add("COMMENT=" + comment1);
         args.add("REORDER_INPUT_BY_FIRST_VARIANT=" +  reorder);
 
-        Assert.assertEquals(runPicardCommandLine(args.toArray(new String[args.size()])), expectedRetVal, "Program was expected to run successfully, but didn't.");
+        Assert.assertEquals(runPicardCommandLine(args.toArray(new String[]{})), expectedRetVal, "Program was expected to run successfully, but didn't.");
 
         if (expectedRetVal == 0) {
             final VCFFileReader expectedReader = new VCFFileReader(expectedOutput, false);
